
include std/iup.e
include std/iup_scintilla.e
include actions.e
include events.e

--**
-- Create a menu item.
--
public function menu_item( sequence handle, sequence text, sequence image, atom action_cb, atom active = 1 )

	atom item = IupItem( text, NULL )
	IupSetAttribute( item, "IMAGE", get_image(image) )
	IupSetInt( item, "ACTIVE", active )
	IupSetCallback( item, "ACTION", action_cb )
	IupSetHandle( handle, item )

	return item
end function

--**
-- Create a menu separator
--
public function menu_separator()
	return IupSeparator()
end function

--**
-- Create a toolbar item.
--
public function toolbar_item( sequence handle, sequence tip, sequence image, atom action_cb, atom active = 1 )

	atom item = IupButton()
	IupSetAttributes( item, "CANFOCUS=NO, FLAT=YES" )
	IupSetAttribute( item, "IMAGE", get_image(image) )
	IupSetAttribute( item, "TIP", tip )
	IupSetInt( item, "ACTIVE", active )
	IupSetCallback( item, "ACTION", action_cb )
	IupSetHandle( handle, item )

	return item
end function

--**
-- Create a toolbar separator.
--
public function toolbar_separator()
	return IupSetAttributes( IupLabel(NULL), "SEPARATOR=VERTICAL" )
end function

--**
-- Create the popup menu.
--
public function popup_menu()

	atom menu = IupMenu({
		menu_item( "popup_new",    "&New\t\tCtrl+N",   "IUP_FileNew",     Icallback("new_cb"),    1 ),
		menu_item( "popup_open",   "&Open...\tCtrl+O", "IUP_FileOpen",    Icallback("open_cb"),   1 ),
		menu_item( "popup_save",   "&Save\t\tCtrl+S",  "IUP_FileSave",    Icallback("save_cb"),   0 ),
		menu_item( "popup_saveas", "Save &As..",       "IUP_FileSaveAs",  Icallback("saveas_cb"), 1 ),
		menu_item( "popup_close",  "C&lose\t\tCtrl+W", "IUP_FileClose",   Icallback("close_cb"),  1 )
	})
	IupSetHandle( "popup_menu", menu )

	return menu
end function

--**
-- Create the File menu.
--
public function file_menu()

	atom menu = IupMenu({
		menu_item( "file_new",    "&New\t\tCtrl+N",   "IUP_FileNew",     Icallback("new_cb"),    1 ),
		menu_item( "file_open",   "&Open...\tCtrl+O", "IUP_FileOpen",    Icallback("open_cb"),   1 ),
		menu_item( "file_save",   "&Save\t\tCtrl+S",  "IUP_FileSave",    Icallback("save_cb"),   0 ),
		menu_item( "file_saveas", "Save &As..",       "IUP_FileSaveAs",  Icallback("saveas_cb"), 0 ),
		menu_item( "file_close",  "C&lose\t\tCtrl+W", "IUP_FileClose",   Icallback("close_cb"),  0 ),
		menu_separator(),
		menu_item( "file_exit",   "E&xit\t\tAlt+F4",  "IUP_FileExit", Icallback("exit_cb"), 1 )
	})
	IupSetHandle( "file_menu", menu )

	return menu
end function

--**
-- Create the Edit menu.
--
public function edit_menu()

	atom menu = IupMenu({
		menu_item( "edit_undo", "&Undo\t\tCtrl+Z", "IUP_EditUndo", Icallback("undo_cb"), 0 ),
		menu_item( "edit_redo", "&Redo\t\tCtrl+Y", "IUP_EditRedo", Icallback("redo_cb"), 0 ),
		menu_separator(),
		menu_item( "edit_cut",    "Cu&t\t\tCtrl+X",   "IUP_EditCut",    Icallback("cut_cb"),    0 ),
		menu_item( "edit_copy",   "&Copy\t\tCtrl+C",  "IUP_EditCopy",   Icallback("copy_cb"),   0 ),
		menu_item( "edit_paste",  "&Paste\t\tCtrl+V", "IUP_EditPaste",  Icallback("paste_cb"),  0 ),
		menu_item( "edit_delete", "&Delete\t\tDel",   "IUP_EditDelete", Icallback("delete_cb"), 0 ),
		menu_separator(),
		menu_item( "edit_selectall", "Select &All\tCtrl+A", "IUP_EditSelectAll", Icallback("selectall_cb"), 0 )
	})
	IupSetHandle( "edit_menu", menu )

	return menu
end function

--**
-- Create the View menu.
--
public function view_menu()

	atom menu = IupMenu({
		menu_item( "view_zoomin",  "Zoom &In\tCtrl++",    "IUP_ZoomIn",    Icallback("zoomin_cb")    ),
		menu_item( "view_zoomout", "Zoom &Out\tCtrl+-",   "IUP_ZoomOut",   Icallback("zoomout_cb")   ),
		menu_item( "view_zoom100", "&Reset Zoom\tCtrl+0", "IUP_ZoomReset", Icallback("zoomreset_cb") )
	})
	IupSetHandle( "view_menu", menu )

	return menu
end function

--**
-- Create the Run menu.
--
public function run_menu()

	atom menu = IupMenu({
		menu_item( "run_run",  "&Run\tF5",     "IUP_ActionRun",  Icallback("run_cb")  ),
		menu_item( "run_bind", "&Bind...\tF7", "IUP_ActionBind", Icallback("bind_cb") )
	})
	IupSetHandle( "run_menu", menu )

	return menu
end function

--**
-- Create the Help menu.
--
public function help_menu()

	atom menu = IupMenu({
		menu_item( "help_layout", "Dialog Layout",  "IUP_DialogLayout", Icallback("layout_cb") ),
		menu_separator(),
		menu_item( "help_about",  "&About...\tF12", "IUP_MessageInfo", Icallback("about_cb")  )
	})
	IupSetHandle( "help_menu", menu )

	return menu
end function

--**
-- Create the main menu.
--
public function main_menu()

	atom menu = IupMenu({
		IupSubmenu( "&File", file_menu() ),
		IupSubmenu( "&Edit", edit_menu() ),
		IupSubmenu( "&View", view_menu() ),
		IupSubmenu( "&Run",  run_menu()  ),
		IupSubmenu( "&Help", help_menu() )
	})
	IupSetHandle( "menu", menu )

	return menu
end function

--**
-- Create the main toolbar.
--
public function main_toolbar()

	atom toolbar = IupHbox({
		toolbar_item( "tool_new",    "New (Ctrl+N)",     "IUP_FileNew",    Icallback("new_cb"),    1 ),
		toolbar_item( "tool_open",   "Open (Ctrl+O)",    "IUP_FileOpen",   Icallback("open_cb"),   1 ),
		toolbar_item( "tool_save",   "Save (Ctrl+S)",    "IUP_FileSave",   Icallback("save_cb"),   0 ),
		toolbar_item( "tool_saveas", "Save As",          "IUP_FileSaveAs", Icallback("saveas_cb"), 1 ),
		toolbar_item( "tool_close",  "Close (Ctrl+W)",   "IUP_FileClose",  Icallback("close_cb"),  0 ),
		toolbar_separator(),
		toolbar_item( "tool_cut",   "Cut (Ctrl+X)",   "IUP_EditCut",   Icallback("cut_cb"),   0 ),
		toolbar_item( "tool_copy",  "Copy (Ctrl+C)",  "IUP_EditCopy",  Icallback("copy_cb"),  0 ),
		toolbar_item( "tool_paste", "Paste (Ctrl+V)", "IUP_EditPaste", Icallback("paste_cb"), 0 ),
		toolbar_item( "tool_undo",  "Undo (Ctrl+Z)",  "IUP_EditUndo",  Icallback("undo_cb"),  0 ),
		toolbar_item( "tool_redo",  "Redo (Ctrl+Y)",  "IUP_EditRedo",  Icallback("redo_cb"),  0 ),
		toolbar_separator(),
		toolbar_item( "tool_zoomin",  "Zoom In (Ctrl++)",  "IUP_ZoomIn",  Icallback("zoomin_cb"),  1 ),
		toolbar_item( "tool_zoomout", "Zoom Out (Ctrl+-)", "IUP_ZoomOut", Icallback("zoomout_cb"), 1 ),
		toolbar_separator(),
		toolbar_item( "tool_run",  "Run (F5)",  "IUP_ActionRun",  Icallback("run_cb"),  0 ),
		toolbar_item( "tool_bind", "Bind (F7)", "IUP_ActionBind", Icallback("bind_cb"), 0 ),
		toolbar_separator(),
		toolbar_item( "tool_about", "About (F12)", "IUP_MessageInfo", Icallback("about_cb") )
	})
	IupSetAttributes( toolbar, "GAP=2, MARGIN=2x2" )
	IupSetHandle( "toolbar", toolbar )

	return toolbar
end function

--**
-- Create the main status bar.
--
public function main_statusbar()

	atom field1 = IupLabel()
	IupSetAttribute( field1, "MINSIZE", "0x23" )
	IupSetAttribute( field1, "NAME", "STATUSBAR" )
	IupSetAttribute( field1, "EXPAND", "HORIZONTAL" )
	IupSetAttribute( field1, "PADDING", "2x4" )
	IupSetHandle( "field1", field1 )

	atom field2 = IupLabel()
	IupSetAttribute( field2, "MINSIZE", "100x23" )
	IupSetAttribute( field2, "NAME", "STATUSBAR" )
--	IupSetAttribute( field2, "EXPAND", "HORIZONTAL" )
	IupSetAttribute( field2, "PADDING", "2x4" )
	IupSetHandle( "field2", field2 )

	atom status = IupHbox({ field1, field2 })
	IupSetHandle( "status", status )

	return status
end function

--**
-- Create the main tabs interface.
--
public function main_tabs()

	atom tabs = IupTabs()
	IupSetAttribute( tabs, "SHOWCLOSE", "YES" )
	IupSetCallback( tabs, "RIGHTCLICK_CB", Icallback("rightclick_cb") )
	IupSetCallback( tabs, "TABCHANGE_CB",  Icallback("tabchange_cb")  )
	IupSetCallback( tabs, "TABCLOSE_CB",   Icallback("tabclose_cb")   )
	IupSetHandle( "tabs", tabs )

	return tabs
end function

--**
-- Create the main dialog.
--
public function main_dialog()

	atom menu = main_menu()
	atom vbox = IupVbox({
		main_toolbar(),
		main_tabs(),
		main_statusbar()
	})

	atom dialog = IupDialog( vbox )
	IupSetAttribute( dialog, "SIZE", "600x400" )
	IupSetAttribute( dialog, "MENU", "menu" )
	IupSetCallback( dialog, "CLOSE_CB", Icallback("closing_cb") )
	IupSetHandle( "dialog", dialog )

	atom timer = IupTimer()
	IupSetAttribute( timer, "TIME", "100" )
	IupSetAttribute( timer, "RUN", "YES" )
	IupSetCallback( timer, "ACTION_CB", Icallback("timer_cb") )
	IupSetHandle( "timer", timer )

	return dialog
end function

