--****
-- == Euphoria Editor
--
-- by Gregory Haberek
-- <ghaberek@gmail.com>
--
-- === Version 0.1b
-- 25 October 2015
-- * Added icons to menu options
-- * Added icons to tab to show status
-- * Added bind utility dialog to Run menu
-- * Added save-as and close buttons to toolbar
-- * Added right-click menu to tab interface
-- * Added zoom functions to menu and toolbar
-- * Changed to improved icon set from Fatcow
-- * Fixed missing eu.cfg in bin directory
-- * Fixed text styles missing after save
-- * Fixed F5 launching behavior
--
-- === Version 0.1a
-- 18 October 2015
-- * Initial release
--

include std/iup.e
include std/iup_config.e
include std/iup_scintilla.e
include actions.e
include controls.e
include events.e

constant APP_NAME    = "euphoria_editor"
constant APP_TITLE   = "Euphoria Editor"
constant APP_VERSION = "Version 0.1a"
constant APP_RELEASE = "18 October 2015"
constant APP_AUTHOR  = "Gregory Haberek"
constant APP_EMAIL   = "ghaberek@gmail.com"
constant APP_WEBSITE = "http://bitbucket.org/ghaberek/editor"

--**
-- The main application loop.
--
procedure main( sequence cmd = command_line() )

	sequence argv = cmd[3..$]
	integer argc = length( argv )

	IupOpen( argc, argv )
	IupImageLibOpen()
	IupScintillaOpen()

	atom config = IupConfig()
	IupSetAttribute( config, "APP_NAME", APP_NAME )
	IupConfigLoad( config )
	IupSetHandle( "config", config )

	atom dialog = main_dialog()
	IupSetAttribute( dialog, "TITLE",   APP_TITLE   )
	IupSetAttribute( dialog, "VERSION", APP_VERSION )
	IupSetAttribute( dialog, "RELEASE", APP_RELEASE )
	IupSetAttribute( dialog, "AUTHOR",  APP_AUTHOR  )
	IupSetAttribute( dialog, "EMAIL",   APP_EMAIL   )
	IupSetAttribute( dialog, "WEBSITE", APP_WEBSITE )
	register_hotkeys()

	IupShowXY( dialog, IUP_CENTER, IUP_CENTER )

	-- create an empty default tab
	atom sci = add_tab()
	apply_lexer( sci )

	IupMainLoop()

	IupClose()

end procedure

main()

