
include std/iup.e
include std/iup_scintilla.e
include std/cmdline.e
include std/console.e
include std/filesys.e
include std/io.e
include std/machine.e
include std/pipeio.e
include std/sequence.e
include actions.e
include controls.e

integer m_skip = 0

--**
-- N.B. THIS IS A BUG WORK-AROUND!
--
-- Keypress events (e.g. Ctrl+N) are somtimes triggered twice.
-- We use a timer to prevent our hotkey-enabled callbacks from
-- being called too many times within a small timeframe (50ms).
--
function skip_next_event()

	integer old_skip = m_skip

	ifdef LINUX then
		m_skip = not m_skip
	end ifdef

	return old_skip
end function

--**
-- Create a new empty document.
--
public function new_cb( atom self )
--	IupAlarm( "Action", "New", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom sci = add_tab( "Untitled" )
	IupSetAttribute( sci, "PATH", "" )
	apply_lexer( sci )
	
	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetChildPos( tabs, sci )
	IupSetAttributeId( tabs, "TABIMAGE", pos, get_image("IUP_FileNew") )

	atom field1 = IupGetHandle( "field1" )
	IupSetAttribute( field1, "TITLE", "" )

	return IUP_IGNORE
end function

--**
-- Open an existing document.
--
public function open_cb( atom self )
--	IupAlarm( "Action", "Open", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom dlg = IupFileDlg()
	IupSetAttribute( dlg, "DIALOGTYPE", "OPEN" )
	IupSetAttribute( dlg, "TITLE", "Open" )
	IupSetAttribute( dlg, "FILTER", "*.*" )
	IupSetAttribute( dlg, "FILTERINFO", "All files (*.*)" )

	IupPopup( dlg, IUP_CURRENT, IUP_CURRENT )
	if IupGetInt( dlg, "STATUS" ) = IUP_INVALID then
		return IUP_IGNORE
	end if

	sequence path = IupGetAttribute( dlg, "VALUE" )
	sequence name = filesys:filename( path )
	sequence text = read_file( path )

	atom tabs = IupGetHandle( "tabs" )
	atom count = IupGetChildCount( tabs )
	if count = 1 then
		
		sequence title = IupGetAttributeId( tabs, "TABTITLE", 0 )
		if equal( title, "Untitled" ) then
			-- remove the empty unmodified tab
			atom sci = IupGetChild( tabs, 0 )
			IupDetach( sci )
		end if
		
	end if

	atom sci = add_tab( name, text )
	IupSetAttribute( sci, "PATH", path )
	apply_lexer( sci, path )
	
	atom pos = IupGetChildPos( tabs, sci )
	IupSetAttributeId( tabs, "TABIMAGE", pos, get_image("IUP_FileSave") )

	atom field1 = IupGetHandle( "field1" )
	IupSetAttribute( field1, "TITLE", path )

	return IUP_IGNORE
end function

--**
-- Save the currently selected document.
--
public function save_cb( atom self )
--	IupAlarm( "Action", "Save", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )
	sequence modified = IupGetAttribute( sci, "SAVEDSTATE" )
	if equal( modified, "NO" ) then
		-- nothing to save
		return IUP_IGNORE
	end if

	sequence path = IupGetAttribute( sci, "PATH" )
	if length( path ) = 0 then
		-- file has not been named yet

		atom dlg = IupFileDlg()
	IupSetAttribute( dlg, "DIALOGTYPE", "SAVE" )
	IupSetAttribute( dlg, "TITLE", "Save" )
	IupSetAttribute( dlg, "FILTER", "*.*" )
	IupSetAttribute( dlg, "FILTERINFO", "All files (*.*)" )

		IupPopup( dlg, IUP_CURRENT, IUP_CURRENT )
		if IupGetInt( dlg, "STATUS" ) = IUP_INVALID then
			-- user cancelled
			return IUP_IGNORE
		end if

		path = IupGetAttribute( dlg, "VALUE" )
		IupSetAttribute( sci, "PATH", path )
		IupSetAttributeId( tabs, "TABIMAGE", pos, get_image("IUP_FileSave") )
		apply_lexer( sci )

	end if

	sequence text = IupGetAttribute( sci, "VALUE" )
	io:write_file( path, text )

	sequence name = filesys:filename( path )
	IupSetAttributeId( tabs, "TABTITLE", pos, name )
	IupSetAttribute( sci, "SAVEDSTATE" )

	return IUP_IGNORE
end function

--**
-- Save the document with a new name.
--
public function saveas_cb( atom self )
--	IupAlarm( "Action", "Save As", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom dlg = IupFileDlg()
	IupSetAttribute( dlg, "DIALOGTYPE", "SAVE" )
	IupSetAttribute( dlg, "TITLE", "Save As" )
	IupSetAttribute( dlg, "FILTER", "*.*" )
	IupSetAttribute( dlg, "FILTERINFO", "All files (*.*)" )

	IupPopup( dlg, IUP_CURRENT, IUP_CURRENT )
	if IupGetInt( dlg, "STATUS" ) = IUP_INVALID then
		-- user cancelled
		return IUP_IGNORE
	end if

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )

	sequence path = IupGetAttribute( dlg, "VALUE" )
	IupSetAttribute( sci, "PATH", path )
	IupSetAttributeId( tabs, "TABIMAGE", pos, get_image("IUP_FileSave") )
	apply_lexer( sci )

	sequence text = IupGetAttribute( sci, "VALUE" )
	io:write_file( path, text )

	sequence name = filesys:filename( path )
	IupSetAttributeId( tabs, "TABTITLE", pos, name )
	IupSetAttribute( sci, "SAVEDSTATE" )
	IupRefresh( tabs )

	return IUP_IGNORE
end function

--**
-- Close the currently selected document.
--
public function close_cb( atom self )
--	IupAlarm( "Action", "Close", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )
	atom modified = IupGetInt( sci, "SAVEDSTATE" )

	if modified then
		-- ask the user to save

		sequence name = "Untitled"
		sequence path = IupGetAttribute( sci, "PATH" )

		if length( path ) then
			name = filesys:filename( path )
		end if

		atom response = confirm_close( name )

		if response = 1 then
			-- user wants to save

			save_cb( self )
			modified = IupGetInt( sci, "SAVEDSTATE" )
			if modified then
				-- user cancelled save
				return IUP_IGNORE
			end if

		elsif response = 3 then
			-- user cancelled prompt
			return IUP_IGNORE

		end if

	end if

	-- remove the tab
	IupDestroy( sci )
	IupRefreshChildren( tabs )

	atom count = IupGetChildCount( tabs )
	if count != 0 then
		-- update the tab selection

		while pos >= count do
			-- pos is out of bounds
			pos -= 1
		end while

		sci = IupGetChild( tabs, pos )
		IupSetAttributeHandle( tabs, "VALUE", sci )

		IupRefresh( tabs )
		IupSetFocus( sci )

	end if

	return IUP_IGNORE
end function

--**
-- Exit the application.
--
public function exit_cb( atom self )
--	IupAlarm( "Action", "Exit", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	-- tell the application to close
	return IUP_CLOSE
end function

--**
-- Undo the last bufferred action.
--
public function undo_cb( atom self )
--	IupAlarm( "Action", "Undo", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )
	if IupGetInt( sci, "UNDO" ) then
		IupSetAttribute( sci, "UNDO", "YES" )
	end if

	return IUP_IGNORE
end function

--**
-- Redo the last buffered action.
--
public function redo_cb( atom self )
--	IupAlarm( "Action", "Redo", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )
	if IupGetInt( sci, "REDO" ) then
		IupSetAttribute( sci, "REDO", "YES" )
	end if

	return IUP_IGNORE
end function

--**
-- Cut the currently selected text.
--
public function cut_cb( atom self )
--	IupAlarm( "Action", "Cut", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "CLIPBOARD", "CUT" )

	return IUP_IGNORE
end function

--**
-- Copy the currently selected text.
--
public function copy_cb( atom self )
--	IupAlarm( "Action", "Copy", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "CLIPBOARD", "COPY" )

	return IUP_IGNORE
end function

--**
-- Paste the contents of the clipboard.
--
public function paste_cb( atom self )
--	IupAlarm( "Action", "Paste", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "CLIPBOARD", "PASTE" )

	return IUP_IGNORE
end function

--**
-- Delete the currently selected text.
--
public function delete_cb( atom self )
--	IupAlarm( "Action", "Delete", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "CLIPBOARD", "CLEAR" )

	return IUP_IGNORE
end function

--**
-- Select all text in the current document.
--
public function selectall_cb( atom self )
--	IupAlarm( "Action", "Select All", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	atom count = IupGetInt( sci, "COUNT" )
	sequence selection = sprintf("%d:%d", {0,count} )
	IupSetAttribute( sci, "SELECTIONPOS", selection )

	return IUP_IGNORE
end function

--**
-- Increase the zoom in the current document.
--
public function zoomin_cb( atom self )
--	IupAlarm( "Action", "Zoom In", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "ZOOMIN", "YES" )

	return IUP_IGNORE
end function

--**
-- Decrease the zoom in the current document.
--
public function zoomout_cb( atom self )
--	IupAlarm( "Action", "Zoom Out", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "ZOOMOUT", "YES" )

	return IUP_IGNORE
end function

--**
-- Reset the zoom in the current document.
--
public function zoomreset_cb( atom self )
--	IupAlarm( "Action", "Reset Zoom", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	IupSetAttribute( sci, "ZOOM", "0" )

	return IUP_IGNORE
end function

--**
-- Run the file in the current tab.
--
public function run_cb( atom self )
--	IupAlarm( "Action", "Run", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	sequence path = IupGetAttribute( sci, "PATH" )
	sequence name = filesys:filename( path )

	ifdef WINDOWS then
		sequence comspec = getenv( "ComSpec" )
		sequence eui = locate_file( "euiw.exe" )
		sequence cmd = build_commandline({ comspec, "/C", eui, name })
	elsedef
		sequence xterm = "/usr/bin/x-terminal-emulator"
		sequence eui = locate_file( "eui" )
		sequence cmd = build_commandline({ xterm, "-e", eui, name })
	end ifdef

	chdir( filesys:pathname(path) )
	pipeio:exec( cmd, pipeio:create() )

	return IUP_IGNORE
end function

--**
-- Run the bind utility.
--
public function bind_cb( atom self )
--	IupAlarm( "Action", "Bind", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetInt( tabs, "VALUEPOS" )
	if pos = IUP_INVALID then
		-- nothing selected
		return IUP_IGNORE
	end if

	atom sci = IupGetChild( tabs, pos )
	sequence path = IupGetAttribute( sci, "PATH" )

	ifdef WINDOWS then
		sequence eubind = "eubind.exe"
		sequence backend = locate_file( "eub.exe" )
	elsedef
		sequence eubind = "eubind"
		sequence backend = locate_file( "eub" )
	end ifdef

	-- build the input and output files from the original path
	sequence input  = pathname( path ) & SLASH & filename( path )
	sequence output = pathname( path ) & SLASH & filebase( path )
	ifdef WINDOWS then output &= ".exe" end ifdef

	-- an empty string to pad memory
	sequence empty = repeat( 0, 255 )

	sequence fields = {
		"Bind     %t",
		"Input:   %f[OPEN|*.e;*.ew;*.ex;*.exw|CURRENT|NO|NO]",
		"Output:  %f[SAVE|*.exe|CURRENT|NO|YES]",
		"Backend: %f[OPEN|*.exe|CURRENT|NO|NO]",
		"Options  %t",
		"Icon:    %f[OPEN|*.ico|CURRENT|NO|NO]",
		"         %b[Include debug symbols,Include debug symbols]",
		"         %b[Create console application,Create console application]"
	}

	sequence params = {
		allocate_string( empty, 1 ), -- Input
		allocate_string( empty, 1 ), -- Output
		allocate_string( empty, 1 ), -- Backend
		allocate_string( empty, 1 ), -- Icon
		allocate_data( 4, 1 ),        -- Include debug symbols
		allocate_data( 4, 1 )         -- Use current console
	}

	poke(  params[1], input )
	poke(  params[2], output )
	poke(  params[3], backend )
	poke4( params[5], 0 ) -- debug = 0
	poke4( params[6], 0 ) -- console = 0

	atom result = IupGetParam( "Bind", NULL, NULL,
		stdseq:join(fields, "\n"), length(fields), 0, params )

	if result = 0 then
		-- user cancelled
		return IUP_IGNORE
	end if

	atom debug, console
	sequence icon
	input   = peek_string( params[1] )
	output  = peek_string( params[2] )
	backend = peek_string( params[3] )
	icon    = peek_string( params[4] )
	debug   = peek4u( params[5] )
	console = peek4u( params[6] )

	-- build the command line
	sequence cmd = { eubind, "-batch" }
	if console != 0      then cmd &= { "-con" }          end if
	if length( backend ) then cmd &= { "-eub", backend } end if
	if debug != 0        then cmd &= { "-full_debug" }   end if
	if length( icon )    then cmd &= { "-icon", icon }   end if
	if length( output )  then cmd &= { "-out", output }  end if
	cmd &= { filesys:filename(input) }
	cmd = build_commandline( cmd )

	chdir( filesys:pathname(input) )
	pipeio:exec( cmd, pipeio:create() )

	return IUP_IGNORE
end function

--**
-- Show the "About" dialog.
--
public function about_cb( atom self )
--	IupAlarm( "Action", "About", "OK" )

	if skip_next_event() then
		return IUP_IGNORE
	end if

	atom dialog = IupGetHandle( "dialog" )
	sequence title   = IupGetAttribute( dialog, "TITLE" )
	sequence version = IupGetAttribute( dialog, "VERSION" )
	sequence release = IupGetAttribute( dialog, "RELEASE" )
	sequence author  = IupGetAttribute( dialog, "AUTHOR" )
	sequence email   = IupGetAttribute( dialog, "EMAIL" )
	sequence website = IupGetAttribute( dialog, "WEBSITE" )

	sequence message = sprintf(
		"%s\n" &    -- title
		"- %s\n" &  -- version
		"- %s\n" &  -- release
		"\n" &
		"by %s\n" & -- author
		"- %s\n" &  -- email
		"\n" &
		"%s",       -- website
		{title,version,release,
		author, email,website}
	)

	atom result = IupAlarm( title, message, "Website", "Close" )
	if result = 1 then
		-- user clicked website button
		IupHelp( website )
	end if

	return IUP_IGNORE
end function

--**
-- Ask the user to save while closing.
--
public function closing_cb( atom self )
--	IupAlarm( "Action", "Closing", "OK" )

	atom tabs = IupGetHandle( "tabs" )

	atom count = IupGetChildCount( tabs )
	for pos = 0 to count - 1 do

		atom sci = IupGetChild( tabs, pos )
		atom modified = IupGetInt( sci, "SAVEDSTATE" )

		if modified then
			-- ask the user to save

			sequence name = "Untitled"
			sequence path = IupGetAttribute( sci, "PATH" )

			if length( path ) then
				name = filesys:filename( path )
			end if

			atom response = confirm_close( name )

			if response = 1 then
				-- user wants to save

				atom result = save_cb( self )
				if result != IUP_IGNORE then
					return IUP_IGNORE
				end if

			elsif response = 3 then
				-- user cancelled
				return IUP_IGNORE

			end if

		end if

	end for

	-- allow he window to close
	return IUP_DEFAULT
end function

--**
-- Show layout dialog (Debugging).
--
public function layout_cb( atom self )
--	IupAlarm( "Action", "Layout", "OK" )

	atom dialog = IupGetHandle( "dialog" )
	atom layout = IupLayoutDialog( dialog )
	IupSetAttribute( layout, "SIZE", "500x300" )
	IupShowXY( layout, IUP_CURRENT, IUP_CURRENT )

	return IUP_IGNORE
end function

--**
-- Select the previous tab when Ctrl+PgUp is pressed.
--
public function pgup_cb( atom self )
--	IupAlarm( "Action", "Page up", "OK" )

	atom tabs = IupGetHandle( "tabs" )
	atom count = IupGetChildCount( tabs )
	if count = 0 then
		-- no tabs
		return IUP_DEFAULT
	end if

	atom pos = IupGetInt( tabs, "VALUEPOS" ) - 1
	if pos < 0 then pos = count - 1 end if
	IupSetInt( tabs, "VALUEPOS", pos )

	atom sci = IupGetChild( tabs, pos )
	IupSetFocus( sci )

	return IUP_IGNORE
end function

--**
-- Select the next tab when Ctrl+PgDn is pressed.
--
public function pgdn_cb( atom self )
--	IupAlarm( "Action", "Page down", "OK" )

	atom tabs = IupGetHandle( "tabs" )
	atom count = IupGetChildCount( tabs )
	if count = 0 then
		-- no tabs
		return IUP_DEFAULT
	end if

	atom pos = IupGetInt( tabs, "VALUEPOS" ) + 1
	if pos > count-1 then pos = 0 end if
	IupSetInt( tabs, "VALUEPOS", pos )

	atom sci = IupGetChild( tabs, pos )
	IupSetFocus( sci )

	return IUP_IGNORE
end function

--**
-- Show popup menu when tab is right clicked.
--
public function rightclick_cb( atom self, atom pos )
--	IupAlarm( "Action", "Right click", "OK" )

	atom sci = IupGetChild( self, pos )
	IupSetAttribute( self, "VALUE", pos )

	atom popup = popup_menu()
	toggle_item( "popup_save", IupGetAttribute(sci, "SAVEDSTATE") )

	sequence curpos = get_cursor_pos()
	IupPopup( popup, curpos[1], curpos[2] )

	IupDestroy( popup )

	return IUP_DEFAULT
end function

--**
-- Set focus to the control when the tab changes.
--
public function tabchange_cb( atom self, atom new_tab, atom old_tab )

	IupSetFocus( new_tab )
	sequence path = IupGetAttribute( new_tab, "PATH" )

	atom field1 = IupGetHandle( "field1" )
	IupSetAttribute( field1, "TITLE", path )

	return IUP_DEFAULT
end function

--**
-- Remove the control when the tab is closed.
--
public function tabclose_cb( atom self, atom pos )
--	IupAlarm( "Action", "Tab close", "OK" )

	atom tabs = IupGetHandle( "tabs" )
	IupSetInt( tabs, "VALUE", pos )

	atom sci = IupGetAttributeHandle( tabs, "VALUE" )
	atom modified = IupGetInt( sci, "SAVEDSTATE" )

	if modified then
		-- ask the user to save

		sequence name = "Untitled"
		sequence path = IupGetAttribute( sci, "PATH" )

		if length( path ) then
			name = filesys:filename( path )
		end if

		atom response = confirm_close( name )

		if response = 1 then
			-- user wants to save

			save_cb( self )
			modified = IupGetInt( sci, "SAVEDSTATE" )
			if modified then
				-- user cancelled save
				return IUP_IGNORE
			end if

		elsif response = 3 then
			-- user cancelled prompt
			return IUP_IGNORE

		end if

	end if

	-- let the tab close
	return IUP_CONTINUE
end function

--**
-- Update the saved status on the tab.
--
public function savepoint_cb( atom self, atom status )
--	IupAlarm( "Action", "Save point", "OK" )

	sequence name = "Untitled"
	sequence path = IupGetAttribute( self, "PATH" )

	if length( path ) then
		name = filesys:filename( path )
	end if

	if not status then name &= "*" end if

	atom tabs = IupGetHandle( "tabs" )
	atom pos = IupGetChildPos( tabs, self )
	IupSetAttributeId( tabs, "TABTITLE", pos, name )
	IupRefresh( tabs )

	return IUP_DEFAULT
end function

--**
-- Reset the skip_next_event() flag.
--
-- Update the menu and toolbar status.
--
public function timer_cb( atom self )
--	IupAlarm( "Action", "Timer", "OK" )

	ifdef LINUX then
		m_skip = 0
	end ifdef

	atom tabs = IupGetHandle( "tabs" )
	atom sci = IupGetAttributeHandle( tabs, "VALUE" )

	-- File menu
	toggle_item( "file_save", IupGetAttribute(sci, "SAVEDSTATE") )
	toggle_item( "file_saveas", (sci != NULL) )
	toggle_item( "file_close",  (sci != NULL) )

	-- Edit menu
	toggle_item( "edit_undo",      IupGetAttribute(sci, "UNDO") )
	toggle_item( "edit_redo",      IupGetAttribute(sci, "REDO") )
	-- --------
	toggle_item( "edit_cut",       IupGetAttribute(sci, "SELECTEDTEXT") )
	toggle_item( "edit_copy",      IupGetAttribute(sci, "SELECTEDTEXT") )
	toggle_item( "edit_paste",     IupGetAttribute(sci, "CLIPBOARD")    )
	toggle_item( "edit_delete",    IupGetAttribute(sci, "SELECTEDTEXT") )
	-- --------
	toggle_item( "edit_selectall", IupGetAttribute(sci, "COUNT") )

	-- Run menu
	toggle_item( "run_run",  IupGetAttribute(sci, "PATH") )
	toggle_item( "run_bind", IupGetAttribute(sci, "PATH") )

	-- Toolbar
	toggle_item( "tool_save",   IupGetAttribute(sci, "SAVEDSTATE") )
	toggle_item( "tool_saveas", (sci != NULL) )
	toggle_item( "tool_close",  (sci != NULL) )
	-- --------
	toggle_item( "tool_cut",    IupGetAttribute(sci, "SELECTEDTEXT") )
	toggle_item( "tool_copy",   IupGetAttribute(sci, "SELECTEDTEXT") )
	toggle_item( "tool_paste",  IupGetAttribute(sci, "CLIPBOARD")    )
	toggle_item( "tool_undo",   IupGetAttribute(sci, "UNDO")         )
	toggle_item( "tool_redo",   IupGetAttribute(sci, "REDO")         )
	-- --------
	toggle_item( "tool_zoomin",  (sci != NULL) )
	toggle_item( "tool_zoomout", (sci != NULL) )
	-- --------
	toggle_item( "tool_run",    IupGetAttribute(sci, "PATH") )
	toggle_item( "tool_bind",   IupGetAttribute(sci, "PATH") )

	return IUP_DEFAULT
end function

